---------------------------------------------------------------
--                                                           
--  RAPID - Rapid Ada Portable Interface Designer     
--                                                           
--  COMMON_DIALOGS.ADB 
--  Description : Implementation of Common Dialogs in Tcl/Tk
--                       
--  By: Martin Carlisle
--                                         
-- RAPID is free software; you can redistribute it and/or 
-- modify it without restriction.  However, we ask that you
-- please retain the original author information, and clearly  
-- indicate if it has been modified.
--
-- RAPID is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
--
---------------------------------------------------------------
with Tcl.Ada;
with Tcl_Utilities;

package body Common_Dialogs is

   procedure Ok_Box(Message : in String;
                 Interp : Tcl.Tcl_Interp;
                 Parent_Window : in String := ".") is

   begin -- Ok_Box
      Tcl.Ada.Tcl_Eval(Interp,"tk_messageBox -message """ &
         Tcl_Utilities.Fix_Quotes(Message) & """ -parent " & Parent_Window);
   end Ok_Box;

   procedure Quit_Dialog(Interp : Tcl.Tcl_Interp;
         Verify : Boolean := True) is
   begin
      if Verify then
         Tcl.Ada.Tcl_Eval(Interp,"set result [" &
            "tk_messageBox -parent . " &
            "-message """ &
            "Do you really want to quit?"" " &
            "-icon warning -title {Exit?} -type yesno]");
         Tcl.Ada.Tcl_Eval(Interp,"if { $result == ""yes"" } " &
            "{ exit }");
      else
         Tcl.Ada.Tcl_Eval(Interp,"exit");
      end if;
   end Quit_Dialog;

   function Yesno_Dialog(Interp : Tcl.Tcl_Interp;
         Message       : in String;
         Title         : in String;
         Parent_Window : in String := ".") return Boolean is
      Result      : String(1..5);
      Result_Last : Integer;
   begin
      Tcl.Ada.Tcl_Eval(Interp,"set result [" &
         "tk_messageBox -parent " & Parent_Window &
         " -message """ &
         Tcl_Utilities.Fix_Quotes(Message) & '"' &
         " -icon question -title {""" & Title & """} -type yesno]");
      Tcl_Utilities.Get_Value(Interp => Interp,
         Name   => "result",
         Result => Result,
         Last   => Result_last);
      if Result_Last = 3 then
         return True;
      else
         return False;
      end if;
   end Yesno_Dialog;


   type File_Dialog_Type is (Open,Save);
   procedure Do_File_Dialog(Interp : Tcl.Tcl_Interp;
      File_Types : in String;
      Filename   : out String;
      File_Last  : out Natural;
      Directory  : out String;
      Dir_Last   : out Natural;
      Title      : in String;
      Initial_Directory : in String;
      Dialog_Type : File_Dialog_Type;
      Default_Extension : in String;
      Change_Directory  : in Boolean) is
      Dialog_Command : String(1..14);
   begin
      if Dialog_Type = Open then
         Dialog_Command := "tk_getOpenFile";
      else
         Dialog_Command := "tk_getSaveFile";
      end if;
      Tcl.Ada.Tcl_Eval(Interp,
         "set file_types { " & File_Types & " }");
      Tcl.Ada.Tcl_Eval(Interp,
         "set filename [" & Dialog_Command &
         " -filetypes $file_types" &
         " -initialdir """ & Initial_Directory & """" &
         " -defaultextension """ & Default_Extension & """" &
         " -title """ & Title & """" &
         " -parent .]");
      Tcl.Ada.Tcl_Eval(Interp,
         "set initialdir [file dirname $filename]");
      if Change_Directory then
         Tcl.Ada.Tcl_Eval(Interp,
            "cd $initialdir");
      end if;
      Tcl_Utilities.Get_Value(Interp => Interp,
         Name   => "filename",
         Result => Filename,
         Last   => File_last);
      Tcl_Utilities.Get_Value(Interp => Interp,
         Name   => "initialdir",
         Result => Directory,
         Last   => Dir_last);
   end Do_File_Dialog;

   -- Ask user for filename to open
   -- File_Types should look like :
   --   { "Tcl_Files" { .tcl .tk } }
   --   { "All Files" * }
   procedure Open_Dialog(Interp : Tcl.Tcl_Interp;
      File_Types : in String;
      Filename   : out String;
      File_Last  : out Natural;
      Title      : in String := "Open";
      Initial_Directory : in String := ".";
      Default_Extension : in String := "";
      Change_Directory  : in Boolean := True) is
      Directory : String(1..1024);
      Dir_Last  : natural;
   begin
      Open_Dialog(Interp,File_Types,Filename,File_Last,Directory,
         Dir_Last, Title, Initial_Directory,
         Default_Extension,Change_Directory);
   end Open_Dialog;

   procedure Open_Dialog(Interp : Tcl.Tcl_Interp;
      File_Types : in String;
      Filename   : out String;
      File_Last  : out Natural;
      Directory  : out String;
      Dir_Last   : out Natural;
      Title      : in String := "Open";
      Initial_Directory : in String := ".";
      Default_Extension : in String := "";
      Change_Directory  : in Boolean := True) is
   begin
      Do_File_Dialog(Interp,File_Types,Filename,File_Last,
         Directory,Dir_Last,Title,Initial_Directory,Open,
         Default_Extension,Change_Directory);
   end Open_Dialog;

   -- Ask user for filename to Save
   -- File_Types should look like :
   --   { "Tcl_Files" { .tcl .tk } }
   --   { "All Files" * }
   procedure Save_Dialog(Interp : Tcl.Tcl_Interp;
      File_Types : in String;
      Filename   : out String;
      File_Last  : out Natural;
      Title      : in String := "Save As";
      Initial_Directory : in String := ".";
      Default_Extension : in String := "";
      Change_Directory  : in Boolean := True) is
      Directory : String(1..1024);
      Dir_Last  : natural;
   begin
      Save_Dialog(Interp,File_Types,Filename,File_Last,Directory,
         Dir_Last, Title, Initial_Directory,
         Default_Extension,Change_Directory);
   end Save_Dialog;

   procedure Save_Dialog(Interp : Tcl.Tcl_Interp;
      File_Types : in String;
      Filename   : out String;
      File_Last  : out Natural;
      Directory  : out String;
      Dir_Last   : out Natural;
      Title      : in String := "Save As";
      Initial_Directory : in String := ".";
      Default_Extension : in String := "";
      Change_Directory  : in Boolean := True) is
   begin
      Do_File_Dialog(Interp,File_Types,Filename,File_Last,
         Directory,Dir_Last,Title,Initial_Directory,Save,
         Default_Extension,Change_Directory);
   end Save_Dialog;
end Common_Dialogs;